@@ -1,11 +1,11 @@
 ;;; efc.el -- Emacs Foundation Classes
-;; $Revision: 1.16 $ $Date: 2004/12/10 03:38:04 $ 
+;; $Revision: 1.18 $ $Date: 2005/03/19 03:50:31 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: lisp, tools, classes
 
-;; Copyright (C) 2001, 2002, 2003, 2004 Paul Kinnucan.
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -555,6 +555,243 @@ compiler process.")
       (setq compilation-last-buffer (oref this :buffer)))))
 
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; Collection Class                                                           ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defclass efc-collection ()
+  ((elem-type :initarg :elem-type
+	      :type (or null symbol)
+	      :initform nil
+	      :documentation "Type of element that this collection contains."))
+  "A collection of objects. The collection can be either homogeneous, i.e.,
+composed of elements of one type, or heterogeneous. The ELEM-TYPE property of
+a heterogeneous collection is nil.")
+
+(defmethod efc-coll-type-compatible-p ((this efc-collection) item)
+  "Returns t if ITEM is type-compatible with this collection. An item is
+type-compatible with a collection if the collection is heterogeneous or
+the item's type is the same as the collection's element type."
+  (let ((element-type (oref this elem-type)))
+    (or (eq element-type nil)
+	(typep item element-type))))
+
+(defmethod efc-coll-iterator ((this efc-collection))
+  "Returns an iterator for this collection."
+  (error "Abstract method."))
+
+(defmethod efc-coll-visit ((this efc-collection) visitor)
+  "Maps VISITOR to each element of the collection. VISITOR
+is an object of efc-visitor class."
+  (let ((iter (efc-coll-iterator this)))
+    (while (efc-iter-has-next iter)
+      (efc-visitor-visit visitor (efc-iter-next iter)))))
+
+(defmethod efc-coll-memberp ((this efc-collection) member)
+  "Returns nonil if this contains item."
+  (error "Abstract method."))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; Iterator Class                                                             ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defclass efc-iterator ()
+  ()
+  "Iterates over a collection.")
+
+(defmethod efc-iter-has-next ((this efc-iterator))
+  "Returns nonnil if the iterator has not returned all of the collection's elements."
+  (error "Abstract method."))
+
+(defmethod efc-iter-next ((this efc-iterator))
+  "Return the next element of the collection."
+  (error "Abstract method."))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; Visitor Class                                                              ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defclass efc-visitor ()
+  ()
+  "Visits each member of a collection.")
+
+(defmethod efc-visitor-visit ((this efc-visitor) member)
+  "Visits MEMBER, a member of a collection."
+  (error "Abstract method."))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; List Class                                                                 ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defclass efc-list (efc-collection)
+  ((items  :initarg :items
+	   :type list
+	   :initform nil
+	   :documentation "List of items."))
+  "List of items.")
+
+(defmethod initialize-instance ((this efc-list) &rest fields)
+  "Iterator constructor."
+  (call-next-method))
+
+(defmethod efc-coll-add ((this efc-list) item)
+  "Adds an item to the list."
+  (if (efc-coll-type-compatible-p this item)
+      (oset this items (append (oref this items) (list item)))
+    (error "Tried to add an item of type %s to a list of items of type %s"
+	   (typep item) (oref this elem-type))))
+
+(defmethod efc-coll-iterator ((this efc-list))
+  "Return an iterator for this list."
+  (efc-list-iterator "list iterator" :list-obj this))
+
+
+(defmethod efc-coll-memberp ((this efc-list) item)
+  "Returns nonil if this list contains item."
+  (member item (oref this items)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; List Iterator Class                                                        ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defclass efc-list-iterator (efc-iterator)
+  ((list-obj :initarg :list-obj
+	     :type efc-list
+	     :documentation "List that this iterator iterates.")
+   (list     :type list
+	     :documentation "Lisp list."))
+  "Iterates over a list.")
+		     
+(defmethod initialize-instance ((this efc-list-iterator) &rest fields)
+  "Iterator constructor."
+  (call-next-method)
+  (assert (oref this list-obj))
+  (assert (typep (oref this list-obj) efc-list))
+  (oset this list (oref (oref this list-obj) items)))
+
+(defmethod efc-iter-has-next ((this efc-list-iterator))
+  "Returns true if this iterator has another list item to return."
+  (oref this list))
+
+(defmethod efc-iter-next ((this efc-list-iterator))
+  "Get next item in the list."
+  (let* ((list (oref this list))
+	 (next (car list)))
+    (oset this list (cdr list))
+    next))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; List Set Class                                                             ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defclass efc-list-set (efc-list)
+  ()
+  "List that contains no duplicates.")
+
+
+(defmethod efc-coll-add ((this efc-list-set) item)
+  "Adds an item to a set only if the set does not
+already contain the item."
+  (if (efc-coll-memberp this item)
+      (error "This set already contains %s" item)
+    (call-next-method)))
+
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; Association Class                                                          ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defclass efc-assoc (efc-list)
+  ()
+  "Association")
+
+(defmethod efc-coll-put ((this efc-assoc) key value)
+  "Put an item into the association list."
+  (oset this items (append (oref this items) (list (cons key value)))))
+
+(defmethod efc-coll-get ((this efc-assoc) key)
+  "Get an item from the association list."
+  (cdr (assq  key (oref this items))))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; Association Set Class                                                      ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defclass efc-assoc-set (efc-assoc)
+  ()
+  "Association that contains no duplicate keys.")
+
+(defmethod efc-coll-put ((this efc-assoc-set) key value)
+  "Adds an item to a set only if the set does not
+already contain the item."
+  (if (efc-coll-get this key)
+      (error "This set already contains %s" key)
+    (call-next-method)))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;                                                                            ;; 
+;; Hash Table Class                                                           ;;
+;;                                                                            ;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defclass efc-hash-table (efc-collection)
+  ((table :type hash-table
+	  :documentation "Lisp table object."))
+  "Hash table.")
+
+  
+(defmethod initialize-instance ((this efc-hash-table) &rest fields)
+  "Hash table constructor."
+  (call-next-method)
+  (oset table (make-hash-table)))
+
+(defmethod efc-coll-put ((this efc-hash-table) key value)
+  "Put an item into the table."
+  (if (efc-coll-type-compatible-p this value)
+      (puthash key value (oref this table))      
+    (error "Tried to add an item of type %s to a hash table of items of type %s"
+	   (typep value) (oref this elem-type))))
+
+(defmethod efc-coll-get ((this efc-hash-table) key)
+  "Get an item from the table."
+  (gethash key (oref this table)))
+
+(defmethod efc-coll-visit ((this efc-hash-table) visitor)
+  "Visit each item in the hash table. VISITOR is an instance
+of efc-visitor class."
+  (maphash
+   (lambda (key value)
+     (efc-visitor-visit visitor value))
+   (oref this table)))
+
+(defmethod efc-coll-iterator ((this efc-hash-table))
+  "Return an iterator for this hash table."
+  (efc-list-iterator 
+   "hash table iterator" 
+   :list-obj (let (values)
+	       (maphash
+		(lambda (key value)
+		  (setq values (append values (list value))))
+		(oref this table))
+	       values)))
+
 
 
 (provide 'efc)
@@ -562,6 +799,12 @@ compiler process.")
 
 ;; Change History
 ;; $Log: efc.el,v $
+;; Revision 1.18  2005/03/19 03:50:31  paulk
+;; Define an association set.
+;;
+;; Revision 1.17  2005/03/18 04:53:14  paulk
+;; Define a set of collection classes.
+;;
 ;; Revision 1.16  2004/12/10 03:38:04  paulk
 ;; Fix efc-compiler to make and set a buffer-local version of compilation-finish-function.
 ;; Thanks To David Evers.
