@@ -1,11 +1,11 @@
 ;;; jde-widgets.el -- Custom-style widgets used by the JDE
-;; $Revision: 1.9 $ $Date: 2000/12/18 05:22:46 $ 
+;; $Revision: 1.26 $ $Date: 2004/06/03 02:21:13 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998, 1999 Paul Kinnucan.
+;; Copyright (C) 1997-2004 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -39,6 +39,8 @@
 
 (require 'wid-edit)
 (require 'eieio)
+(require 'efc)
+(jde-require 'tree-widget)
 
 
 ;; ----------------------------------------------------------------------
@@ -182,68 +184,6 @@
   (widget-setup))
 
 
-(defclass jde-dialog ()
-  ((title     :initarg :title
-	      :type string
-	      :initform "JDE Dialog"
-	      :documentation
-	      "Title of dialog")
-   (buf       :initarg :buf
-	      :type buffer
-	      :documentation
-	      "Dialog buffer")
-   )
-  "Super class of JDE dialogs."
-  )
-
-(defmethod initialize-instance ((this jde-dialog) &rest fields)
-  "Constructor for trace methods dialog."
-
-  ;; Call parent initializer.
-  (call-next-method)
-
-  (oset this buf (get-buffer-create (oref this title)))
-  (set-buffer (oref this buf))
-
-  (jde-dialog-create this)
-
-  (widget-put
-   (widget-create 
-    'push-button
-    :notify 
-    (lambda (button &rest ignore) (jde-dialog-ok (widget-get button :dialog)))
-    "Ok")
-   :dialog this)
-
-  (widget-insert "  ")
-
-  (widget-put
-   (widget-create 
-    'push-button
-    :notify (lambda (button &rest ignore) (jde-dialog-cancel (widget-get button :dialog)))
-    "Cancel")
-   :dialog this)
-
-  (use-local-map widget-keymap)
-  (widget-setup)
-  )
-
-(defmethod jde-dialog-create ((this jde-dialog)))
-
-(defmethod jde-dialog-ok ((this jde-dialog))
-  "Invoked when the user clicks the dialog's okay button. The
-default method kills the dialog buffer."
-  (kill-buffer (current-buffer)))
-
-(defmethod jde-dialog-cancel ((this jde-dialog))
-  "Invoked when the user clicks the dialog's Cancel button. The
-default method kills the dialog buffer."
-  (kill-buffer (current-buffer)))
-
-(defmethod jde-dialog-show ((this jde-dialog))
-   (pop-to-buffer (oref this buf)))
-
-
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;; 
@@ -336,51 +276,74 @@ tree."
 	 (var-tag (format "%s %s" var-type var-name)))
     (cond
      ((typep var-value 'jde-dbs-java-udci)
+      (setq var-tag (format "%s [id: %d]" var-tag (oref var-value :id)))
       (if (string= (oref var-value :jtype) "java.lang.String")
 	  (let* ((cmd (jde-dbs-get-string 
 		       "get string"
 		       :process process
 		       :object-id (oref var-value id)))
 		 (str-val (jde-dbs-cmd-exec cmd)))
-	    (list 'jde-widget-tree :tag var-tag :value t
-		  (list 'jde-widget-tree :tag str-val)))
-	(list 'jde-widget-java-obj :tag var-tag
-	      :process process :object-id (oref var-value :id))))
+	    (list 'tree-widget
+                  :tag var-tag
+                  :node-name var-tag
+                  :open (jde-dbo-locals-open-p var-tag)
+                  :value t
+                  (list 'tree-widget :tag str-val)))
+	(list 'jde-widget-java-obj
+              :tag var-tag
+              :node-name var-tag
+              :open (jde-dbo-locals-open-p var-tag)
+	      :process process
+              :object-id (oref var-value :id))))
      ((typep var-value 'jde-dbs-java-array)
-      (list 'jde-widget-java-array :tag var-tag
+      (setq var-tag (format "%s [id: %d]" var-tag (oref var-value :id)))
+      (list 'jde-widget-java-array
+            :tag var-tag
+            :node-name var-tag
+            :open (jde-dbo-locals-open-p var-tag)
 	    :process process :object var-value))
      ((typep var-value 'jde-dbs-java-primitive)
-      (list 'jde-widget-tree :tag var-tag :value t
-	    (list 'jde-widget-tree 
+      (list 'tree-widget
+            :tag var-tag
+            :node-name var-tag
+            :open (jde-dbo-locals-open-p var-tag)
+            :value t
+	    (list 'tree-widget 
 		  :tag (format "%s" (oref var-value value)))))
      ((typep var-value 'jde-dbs-java-null)
-      (list 'jde-widget-tree :tag var-tag :value t
-	    (list 'jde-widget-tree :tag "null")))
+      (list 'tree-widget
+            :tag var-tag
+            :node-name var-tag
+            :open (jde-dbo-locals-open-p var-tag)
+            :value t
+	    (list 'tree-widget :tag "null")))
      (t
       (error "Unidentified type of local variable: %s" var-tag)))))
 
-(defun jde-widget-java-obj-get-fields (tree-widget)
-  (let* ((process (widget-get tree-widget :process))
-	 (object-id (widget-get tree-widget :object-id))
-	 (cmd
-	  (jde-dbs-get-object
-	   (format "get_object %d" object-id)
-	   :process process
-	   :object-id object-id))
-	 (object
-	  (jde-dbs-cmd-exec cmd))
-	 (fields (oref object fields))
-	 field
-	 nodes)
-    (while fields
-      (setq field (car fields) fields (cdr fields)) 
-      (setq field (cdr field))
-      (push 
-       (jde-widget-java-var-to-tree process field)
-       nodes)) 
-    nodes))
+(defun jde-widget-java-obj-get-fields (obj-widget)
+  (if (widget-get obj-widget :args)
+      (widget-get obj-widget :args)
+    (let* ((process (widget-get obj-widget :process))
+	   (object-id (widget-get obj-widget :object-id))
+	   (cmd
+	    (jde-dbs-get-object
+	     (format "get_object %d" object-id)
+	     :process process
+	     :object-id object-id))
+	   (object
+	    (jde-dbs-cmd-exec cmd))
+	   (fields (oref object fields))
+	   field
+	   nodes)
+      (while fields
+	(setq field (car fields) fields (cdr fields)) 
+	(setq field (cdr field))
+	(push 
+	 (jde-widget-java-var-to-tree process field)
+	 nodes)) 
+      nodes)))
 
-(define-widget 'jde-widget-java-obj 'jde-widget-dtree
+(define-widget 'jde-widget-java-obj 'tree-widget
   "A widget that represents a Java object.
 This widget is essentially a tree node whose entries are the fields
 of the corresponding object. The first time the user expands the node,
@@ -391,7 +354,8 @@ user closes and then reopens the node. Use the syntax
 :tag NAME :process PROCESS :object-id OBJ-ID) to create the widget where
 NAME is the object's name, PROCESS is the process in which
 the object exists, and  ID is the debugger id for the object."
-  :node-fcn 'jde-widget-java-obj-get-fields)
+  :dynargs 'jde-widget-java-obj-get-fields
+  :has-children t)
 
 
 (defun jde-widget-java-array-element-to-tree (process element index)
@@ -403,64 +367,77 @@ the object exists, and  ID is the debugger id for the object."
 		       :process process
 		       :object-id (oref element id)))
 		 (str-val (jde-dbs-cmd-exec cmd)))
-	    (list 'jde-widget-tree :tag (format "[%d] %s" index str-val)))
+	    (list 'tree-widget
+                  :tag (format "[%d] %s" index str-val)
+                  :node-name (format "[%d] %s" index str-val)
+                  :open (jde-dbo-locals-open-p (format "[%d] %s" index str-val))
+                  ))
 	(list 'jde-widget-java-obj 
 	      :tag (format "[%d] %s" index (oref element jtype))
-	      :process process :object-id (oref element id))))
+              :node-name (format "[%d] %s" index (oref element jtype))
+              :open (jde-dbo-locals-open-p (format "[%d] %s" index (oref element jtype)))
+	      :process process
+              :object-id (oref element id))))
      ((typep element 'jde-dbs-java-array)
       (list 'jde-widget-java-array 
 	    :tag (format "[%d] %s" index (oref element jtype))
-	    :process process :object element))
+            :node-name (format "[%d] %s" index (oref element jtype))
+            :open (jde-dbo-locals-open-p (format "[%d] %s" index (oref element jtype)))
+	    :process process
+            :object element))
      ((typep element 'jde-dbs-java-primitive)
-      (list 'jde-widget-tree :tag (format "[%d] %s"  index (oref element value))))
+      (list 'tree-widget :tag (format "[%d] %s"  index (oref element value))))
      ((typep element 'jde-dbs-java-null)
-      (list 'jde-widget-tree :tag (format "[%d] null" index)))
+      (list 'tree-widget :tag (format "[%d] null" index)))
      (t
       (error "Unidentified type of object: <%s|%s>" (oref element jtype) 
 	     (oref element id)))))
 
-(defun jde-widget-java-array-get-elements (tree-widget)
-  (let* ((process (widget-get tree-widget :process))
-	 (array (widget-get tree-widget :object))
-	 cmd array-length)
+(defun jde-widget-java-array-get-elements (array-widget)
+  (if (widget-get array-widget :args)
+      (widget-get array-widget :args)
+    (let* ((process (widget-get array-widget :process))
+	   (array (widget-get array-widget :object))
+	   cmd array-length)
 
-    (setq cmd
-	 (jde-dbs-get-array 
-	   (format "get_array_length %d" (oref array id))
-	   :process process
-	   :array array))
-    (jde-dbs-cmd-exec cmd) 
-
-    (setq array-length
-	  (if (slot-boundp array 'length)
-	     (oref array length)
-	    0))
-    
-    (when (> array-length 0)
       (setq cmd
-	  (jde-dbs-get-array 
-	   (format "get_array_elements %d" (oref array id))
-	   :process process
-	   :array array
-	   :index 0
-	   :length array-length))
-      (jde-dbs-cmd-exec cmd)
-      (let ((elements (oref array elements))
-	    element
-	    nodes 
-	    (index 0))
-	(while elements
-	  (setq element (car elements) elements (cdr elements)) 
-	  (setq nodes 
-		(append nodes 
-			(list (jde-widget-java-array-element-to-tree process element index))))
-	  (setq index (1+ index))) 
-	nodes))))
+	    (jde-dbs-get-array 
+	     (format "get_array_length %d" (oref array id))
+	     :process process
+	     :array array))
+      (jde-dbs-cmd-exec cmd) 
 
-(define-widget 'jde-widget-java-array 'jde-widget-dtree
+      (setq array-length
+	    (if (slot-boundp array 'length)
+		(oref array length)
+	      0))
+    
+      (when (> array-length 0)
+	(setq cmd
+	      (jde-dbs-get-array 
+	       (format "get_array_elements %d" (oref array id))
+	       :process process
+	       :array array
+	       :index 0
+	       :length array-length))
+	(jde-dbs-cmd-exec cmd)
+	(let ((elements (oref array elements))
+	      element
+	      nodes 
+	      (index 0))
+	  (while elements
+	    (setq element (car elements) elements (cdr elements)) 
+	    (setq nodes 
+		  (append nodes 
+			  (list (jde-widget-java-array-element-to-tree process element index))))
+	    (setq index (1+ index))) 
+	  nodes)))))
+
+(define-widget 'jde-widget-java-array 'tree-widget
   "A widget that represents a Java array. Clicking on the widget's
 expand button causes the widget to display the values of the array."
-  :node-fcn 'jde-widget-java-array-get-elements)
+  :dynargs 'jde-widget-java-array-get-elements
+  :has-children t)
 
 (defun test-obj ()
   (interactive)
@@ -607,6 +584,70 @@ expand button causes the widget to display the values of the array."
 (provide 'jde-widgets)
 
 ;; $Log: jde-widgets.el,v $
+;; Revision 1.26  2004/06/03 02:21:13  paulk
+;; jde-require tree-widget.
+;;
+;; Revision 1.25  2003/01/12 23:34:51  jslopez
+;; Small, patch to reduce the amount of cases that cause infinite recursion.
+;;
+;; Revision 1.24  2001/12/04 05:27:26  paulk
+;; Moved dialog classes to the efc (Emacs Foundation Classes) package.
+;;
+;; Revision 1.23  2001/11/29 11:41:18  paulk
+;; Option dialog now positions cursor over OK button.
+;;
+;; Revision 1.22  2001/11/28 16:50:28  jslopez
+;; Fixes bug, that would not keep the state
+;; of nodes created by jde-widget-java-array-element-to-tree.
+;;
+;; Revision 1.21  2001/11/28 08:38:02  paulk
+;; Added note about fix for Emacs 21.1 bug.
+;;
+;; Revision 1.20  2001/11/27 21:04:05  jslopez
+;; Modifies jde-widget-java-var-to-tree to keep nodes the states of nodes.(open/close)
+;;
+;; Revision 1.19  2001/11/24 15:23:11  paulk
+;; The jde-option-dialog versions of jde-dialog-ok and jde-dialog-cancel
+;; now delete the other window ony if the current version of Emacs is not
+;; XEmacs and is less than major version 21.
+;;
+;; Revision 1.18  2001/11/24 07:31:56  paulk
+;; The jde-option-dialog version of jde-dialog-ok now deletes the other windows,
+;; if the current version of Emacs is not 21.1.
+;;
+;; Revision 1.17  2001/11/24 07:23:54  paulk
+;; Changed the jde-dialog-ok method of jde-option-dialog to not delete
+;; other windows. This is necessary to avoid a bug in the Emacs 21.1.1
+;; version of exit-recursive-edit. This line should be restored when
+;; the bug is fixed.
+;;
+;; Revision 1.16  2001/10/26 06:36:07  paulk
+;; Changed jde-option-dialog to use recursive-edit mode to emulate a
+;; modal dialog. Now the show dialog method enters recursive edit mode
+;; thereby suspending the currently executing command until the user
+;; selects the OK or Cancel button on the dialog. Selecting the OK button
+;; on the dialog causes the dialog to update its :selection field to the
+;; option chosen by the user. Selecting the Cancel button sets the
+;; :selection field to nil.
+;;
+;; Revision 1.15  2001/08/09 03:37:11  paulk
+;; Fixed error caused by missing :documentatation keyword in jde-option-dialog. Thanks to David Ponce.
+;;
+;; Revision 1.14  2001/08/07 05:31:26  paulk
+;; Added ok-action-args field to jde-option-dialog.
+;;
+;; Revision 1.13  2001/07/31 05:11:52  paulk
+;; ReleaseNotes.txt
+;;
+;; Revision 1.12  2001/05/21 06:44:29  paulk
+;; Added jde-option-dialog class.
+;;
+;; Revision 1.11  2001/04/19 04:37:14  paulk
+;; Converted array and object trees to derive from David Ponce's tree widget.
+;;
+;; Revision 1.10  2001/02/16 04:38:52  paulk
+;; Added (require 'cl) to enable batch compile.
+;;
 ;; Revision 1.9  2000/12/18 05:22:46  paulk
 ;; *** empty log message ***
 ;;
@@ -637,4 +678,3 @@ expand button causes the widget to display the values of the array."
 ;;
 
 ;; End of jde-widgets.el
-    
\ No newline at end of file
